<?php
// $Id: dynamic_formatters.inc,v 1.1.2.5 2011/01/22 22:47:12 dixon Exp $

$plugin = array(
  'schema' => 'dynamic_formatters_presets',
  'access' => 'administer dynamic formatters',
  'create access' => 'create dynamic formatters',

  'menu' => array(
    'menu item' => 'dynamic-formatters',
    'menu title' => 'Dynamic formatters',
    'menu description' => 'Administer dynamic formatter presets.',
  ),

  'title singular' => t('preset'),
  'title plural' => t('presets'),
  'title singular proper' => t('Dynamic formatter preset'),
  'title plural proper' => t('Dynamic formatter presets'),

  'handler' => array(
    'class' => 'dynamic_formatters_ui',
    'parent' => 'ctools_export_ui',
  ),

  'use wizard' => TRUE,

  'form info' => array(
    'order' => array(
      'basic' => t('Settings'),
      'region' => t('Regions'),
      'variant' => t('Variants'),
      'style' => t('Styles'),
    ),
    // We have to add this form specially because it's invisible.
    'forms' => array(
      'move' => array(
        'form id' => 'ctools_export_ui_edit_item_wizard_form',
      ),
    ),
  ),
);

/**
 * Helper function that returns a region form element.
 */
function dynamic_formatter_region_form($preset_name, $region) {
  $required = $region['key'] == 'default' ? TRUE : FALSE;

  return array(
    '#type' => 'fieldset',
    '#title' => $region['title'],
    '#collapsible' => TRUE,
    '#collapsed' => !$required,
    'title' => array(
      '#type' => 'textfield',
      '#title' => t('Region title'),
      '#default_value' => $region['title'],
      '#required' => $required,
    ),
    'key' => array(
      '#type' => 'textfield',
      '#title' => t('Region key'),
      '#default_value' => $region['key'],
      '#required' => $required,
    ),
    'key_old' => array(
      '#type' => 'hidden',
      '#value' => $region['key'],
    ),
  );
}

/**
 * Helper function that returns a variant form element.
 */
function dynamic_formatter_variant_form($preset_name, $region_key, $variant) {
  $required = $variant['key'] == 'default' ? TRUE : FALSE;

  return array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => !$required,
    '#title' => $variant['title'],
    '#prefix' => '<div id="region-' . check_plain($region_key) . '-variant-' . check_plain($variant['key']) . '">',
    '#suffix' => '</div>',
    'title' => array(
      '#type' => 'textfield',
      '#title' => t('Variant title'),
      '#default_value' => $variant['title'],
      '#required' => $required,
    ),
    'key' => array(
      '#type' => 'textfield',
      '#title' => t('Variant key'),
      '#default_value' => $variant['key'],
      '#required' => $required,
    ),
    'key_old' => array(
      '#type' => 'hidden',
      '#value' => $variant['key'],
    ),
  );
}

/**
 * Helper function that returns a style form element.
 */
function dynamic_formatter_style_form($preset_name, $region_key, $variant, $style_name, $style_title) {
  $form = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => $style_title,
  );

  if (!in_array($style_name, array('group', 'default'))) {
    $form['exclude'] = array(
      '#type' => 'checkbox',
      '#title' => t('Exclude from preset'),
      '#default_value' => $variant['styles'][$style_name]['exclude'],
    );
  }

  $form['prefix'] = array(
    '#type' => 'textarea',
    '#title' => t('Prefix'),
    '#default_value' => $variant['styles'][$style_name]['prefix'],
  );

  $form['suffix'] = array(
    '#type' => 'textarea',
    '#title' => t('Suffix'),
    '#default_value' => $variant['styles'][$style_name]['suffix'],
  );

  // Add formatter selection for all styles except the "group" style.
  if ($style_name != 'group') {
    // Fetch all field types.
    $all_field_types = _content_field_types();
    // Fetch all fields that can have dynamic formatters.
    $dynamic_field_types = module_invoke_all('dynamic_formatters_fields');
    foreach ($dynamic_field_types as $field_type) {
      // Find out what formatters that are available for this
      // field type.
      $formatters = array();
      if (is_array($all_field_types[$field_type]['formatters'])) {
        foreach ($all_field_types[$field_type]['formatters'] as $formatter_name => $formatter_info) {
          $formatters[$formatter_name] = $formatter_info['label'];
        }
      }
      $form['formatters'][$field_type] = array(
        '#type' => 'select',
        '#title' => t('Formatter for @field_type', array('@field_type' => $all_field_types[$field_type]['label'])),
        '#options' => $formatters,
        '#default_value' => $variant['styles'][$style_name]['formatters'][$field_type],
      );
    }
  }
  return $form;
}

/**
 * AHAH page callback for adding a new region form element.
 */
function dynamic_formatter_ahah_region_add($preset_name) {
  // Retrieve the unbuilt form from the form cache.
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);

  // Create the new variant and add it to the form.
  $region = array(
    'title' => t('New'),
    'key' => 'new-' . md5(uniqid(rand(), true)),
  );
  $form['regions'][$region['key']] = dynamic_formatter_region_form($preset_name, $region);

  // Set the new form in the cache.
  $form_state['values'] = array();
  form_set_cache($form_build_id, $form, $form_state);

  // Build the new form against the incoming $_POST values so that we can
  // render the new element.
  $form_state = array('submitted' => FALSE);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  $form = form_builder($form_build_id, $form, $form_state);

  // Render our new form element.
  $form_element = $form['regions'][$region['key']];
  $output = theme('status_messages') . drupal_render($form_element);

  // Return JSON data.
  print drupal_json(array('status' => TRUE, 'data' => $output));
  exit;
}

/**
 * AHAH page callback for adding a new variant form element.
 */
function dynamic_formatter_ahah_variant_add($preset_name, $region_key) {
  // Retrieve the unbuilt form from the form cache.
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);

  // Create the new variant and add it to the form.
  $variant = array(
    'title' => t('New'),
    'key' => 'new-' . md5(uniqid(rand(), true)),
  );
  $form['regions'][$region_key]['variants'][$variant['key']] = dynamic_formatter_variant_form($preset_name, $region_key, $variant);

  // Set the new form in the cache.
  $form_state['values'] = array();
  form_set_cache($form_build_id, $form, $form_state);

  // Build the new form against the incoming $_POST values so that we can
  // render the new element.
  $form_state = array('submitted' => FALSE);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  $form = form_builder($form_build_id, $form, $form_state);

  // Render our new form element.
  $form_element = $form['regions'][$region_key]['variants'][$variant['key']];
  $output = theme('status_messages') . drupal_render($form_element);

  // Return JSON data.
  print drupal_json(array('status' => TRUE, 'data' => $output));
  exit;
}
